/* $Id: cpl_image_io_body.h,v 1.36 2006/11/15 15:04:19 yjung Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2004 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Type dependent macros */
#if CPL_CLASS == CPL_CLASS_DOUBLE
#define CPL_TYPE double
#define CPL_TYPE_T CPL_TYPE_DOUBLE
#define CPL_PTYPE PTYPE_DOUBLE
#define CPL_QFITS_BUF dbuf

#elif CPL_CLASS == CPL_CLASS_FLOAT
#define CPL_TYPE float
#define CPL_TYPE_T CPL_TYPE_FLOAT
#define CPL_PTYPE PTYPE_FLOAT
#define CPL_QFITS_BUF fbuf

#elif CPL_CLASS == CPL_CLASS_INT
#define CPL_TYPE int
#define CPL_TYPE_T CPL_TYPE_INT
#define CPL_PTYPE PTYPE_INT
#define CPL_QFITS_BUF ibuf

#else
#undef CPL_TYPE
#undef CPL_TYPE_T
#undef CPL_PTYPE
#undef CPL_QFITS_BUF
#endif

#if CPL_OPERATION == CPL_IMAGE_IO_ALLOC

    case CPL_TYPE_T:
    image = cpl_malloc(sizeof(cpl_image)) ;
    image->nx = nx ;
    image->ny = ny ;
    image->pixels = cpl_calloc(nx * ny, sizeof(CPL_TYPE));
    image->type = CPL_TYPE_T ;
    image->bpm = NULL ;
    break ;

#elif CPL_OPERATION == CPL_IMAGE_IO_WRAP

    cpl_image   *   im ;

    cpl_ensure( pixels != NULL, CPL_ERROR_NULL_INPUT, NULL) ;
    cpl_ensure( nx > 0,         CPL_ERROR_ILLEGAL_INPUT, NULL) ;
    cpl_ensure( ny > 0,         CPL_ERROR_ILLEGAL_INPUT, NULL) ;

    im = cpl_malloc(sizeof(cpl_image)) ;
    im->nx = nx ;
    im->ny = ny ;
    im->pixels = (void *) pixels;
    im->type = CPL_TYPE_T ;
    
    im->bpm = NULL ;

    return im ;

#elif CPL_OPERATION == CPL_IMAGE_IO_GET
    case CPL_TYPE_T: {
        const CPL_TYPE * pi = (CPL_TYPE*)image->pixels;
        value = (double)pi[index];
        break ;
    }
#elif CPL_OPERATION == CPL_IMAGE_IO_SET
    case CPL_TYPE_T: {
        CPL_TYPE * pi = (CPL_TYPE*)image->pixels;
        pi[index] = value;
        break ;
    }
#elif CPL_OPERATION == CPL_IMAGE_IO_GET_PIXELS

    /* Test entries */
    cpl_ensure(img, CPL_ERROR_NULL_INPUT, NULL);
    cpl_ensure(img->type == CPL_TYPE_T, CPL_ERROR_TYPE_MISMATCH, NULL);

    return (CPL_TYPE*)img->pixels ;

#elif CPL_OPERATION == CPL_IMAGE_IO_COPY

    case CPL_TYPE_T:
        dest = cpl_image_new(src->nx, src->ny, CPL_TYPE_T) ;
        memcpy(dest->pixels, src->pixels, src->nx * src->ny * sizeof(CPL_TYPE));
        break ;

#elif CPL_OPERATION == CPL_IMAGE_IO_LOAD
       
    case CPL_TYPE_T:
        /* Initialize pixel loader */
        ql.ptype    = CPL_PTYPE ;

        cpl_ensure(qfitsloader_init(&ql) == 0, CPL_ERROR_FILE_IO, NULL);
        cpl_ensure(qfits_loadpix(&ql) == 0, CPL_ERROR_BAD_FILE_FORMAT, 
                NULL);

        im = cpl_malloc(sizeof(cpl_image));
        im->nx = ql.lx ;
        im->ny = ql.ly ;
        im->type = CPL_TYPE_T ;
        im->pixels = (void*)ql.CPL_QFITS_BUF ;
        im->bpm = NULL;
        break ;

#elif CPL_OPERATION == CPL_IMAGE_IO_LOAD_WIN
       
    case CPL_TYPE_T:
        /* Initialize pixel loader */
        ql.ptype    = CPL_PTYPE ;

        cpl_ensure(qfitsloader_init(&ql) == 0, CPL_ERROR_FILE_IO, NULL);
        cpl_ensure(qfits_loadpix_window(&ql, llx, lly, urx, ury) == 0, 
                CPL_ERROR_BAD_FILE_FORMAT, NULL);

        im = cpl_malloc(sizeof(cpl_image));
        im->nx = urx - llx + 1 ;
        im->ny = ury - lly + 1 ;
        im->type = CPL_TYPE_T ;
        im->pixels = (void*)ql.CPL_QFITS_BUF ;
        im->bpm = NULL;
        break ;

#elif CPL_OPERATION == CPL_IMAGE_IO_SET_BADPIXEL

  case CPL_TYPE_T: {
      CPL_TYPE          *   pi;
      const CPL_TYPE        acast = (CPL_TYPE) a;
      int                   i ; 
      pi = (CPL_TYPE*)im->pixels ;
      for (i=0 ; i<im->nx*im->ny ; i++) 
          if (pbpm[i] == CPL_BINARY_1) pi[i] = acast ;
      break ;
  }


#elif CPL_OPERATION == CPL_IMAGE_IO_SAVE
        case CPL_TYPE_T:
            qd.ptype     = CPL_PTYPE;
            qd.CPL_QFITS_BUF  = (CPL_TYPE*)outima->pixels ;
            break ;

#endif

#undef CPL_TYPE
#undef CPL_TYPE_T
#undef CPL_PTYPE
#undef CPL_QFITS_BUF
