/* $Id: cpl_frameset.h,v 1.12 2006/05/15 14:55:33 rpalsa Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2005 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: rpalsa $
 * $Date: 2006/05/15 14:55:33 $
 * $Revision: 1.12 $
 * $Name: cpl-3_1 $
 */

#ifndef CPL_FRAMESET_H
#define CPL_FRAMESET_H

#include <cpl_macros.h>
#include <cpl_frame.h>


CPL_BEGIN_DECLS

/**
 * @ingroup cpl_frameset
 *
 * @brief
 *   The frame set data type.
 *
 * This data type is opaque.
 */

typedef struct _cpl_frameset_ cpl_frameset;


/*
 * Create, copy and destroy operations
 */

cpl_frameset *cpl_frameset_new(void);
cpl_frameset *cpl_frameset_duplicate(const cpl_frameset *other);
void cpl_frameset_delete(cpl_frameset *self);

/*
 * Non modifying operations
 */

int cpl_frameset_get_size(const cpl_frameset *self);
int cpl_frameset_is_empty(const cpl_frameset *self);
int cpl_frameset_count_tags(const cpl_frameset *self, const char *tag);

/*
 * Search operations
 */

cpl_frame *cpl_frameset_find(const cpl_frameset *self, const char *tag);

/*
 * Sequential access
 */

cpl_frame *cpl_frameset_get_first(const cpl_frameset *self);
cpl_frame *cpl_frameset_get_next(const cpl_frameset *self);

/*
 * Inserting and removing elements
 */

cpl_error_code cpl_frameset_insert(cpl_frameset *self, cpl_frame *frame);
int cpl_frameset_erase(cpl_frameset *self, const char *tag);
cpl_error_code cpl_frameset_erase_frame(cpl_frameset *self, cpl_frame *frame);

/*
 * Element access
 */

cpl_frame *cpl_frameset_get_frame(const cpl_frameset *self, int position);

/*
 * Miscellaneous functions
 */

int *cpl_frameset_labelise(cpl_frameset *set,
                           int (*compare)(cpl_frame *, cpl_frame *),
                           int *nb_labels);
cpl_frameset *cpl_frameset_extract(cpl_frameset *set, int *labels,
                                   int desired_label);

CPL_END_DECLS

#endif /* CPL_FRAMESET_H */
